function J_sym = ref_loc_cost_function(M,N,params)
% my_cost_function(M,N,params) returns the symbolic cost function J_sym to
% be minimized in order to estimate the variables x_1, ..., x_N from the
% observations c_1, ..., c_M.
%
% M      = number of observations;
% N      = number of variables to be estimated;
% params = any type of vector/matrix/structure containing parameters
%          which should be passed to the cost function;
% J_sym  = returned symbolic cost function;
%

% Symbols for the variables to be estimated
for m=1:M
    eval(['syms x_' num2str(m)]);
end

% Symbols for the observations
for n=1:N
    eval(['syms c_' num2str(n)]);
end

% Here the parameters are the microphone positions, stored in the vector:
% params.mic_pos --> (x,y) coordinates, [N x 2] vector

% Cost function
J_sym = 0;

for n = 1:N
    
    % microphone positions (read from 'params')
    xm = params.mic_pos(n,1); % current x coordinate
    ym = params.mic_pos(n,2); % current y coordinate
            
    % reflective TOA (corresponding to the observation c_n)
    t = eval(['c_' num2str(n)]);
                                             
    % compute the dual ellipse parameters                
    a = -4*t^2*(- t^2 + xm^2 + ym^2)*(- t^2 + xm^2 + ym^2);
    b = 0;
    c = a;
    f = -16*t^2*(- t^2 + xm^2 + ym^2);
    d = f*xm;
    e = f*ym;
 
    % construct the dual ellipse matrix
    CS = [a b/2 d/2;b/2 c e/2;d/2 e/2 f];
    
    % update the cost function
    temp = ([x_1 x_2 1]*CS*[x_1;x_2;1])^2;
    J_sym = J_sym + simplify(temp); % simplify speeds up the computation 
                                    % at the next steps
                  
end